package de.fitko.fitconnect.api.services;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.exceptions.*;

import java.io.File;
import java.util.List;
import java.util.UUID;

/**
 * A technical system that creates a submission via the FIT-Connect Submission API.
 * <p>
 * The Sender acts as a common interface wrapping all client functionality to authenticate <p>
 * and create a valid {@link SubmissionSubmit} including encrypted {@link Data} and {@link Attachment}s
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/sending/overview">Sending Submissions</a>
 */
public interface Sender {

    /**
     * Validates the integrity of the public key including its chain.
     *
     * @param publicKey to validate
     * @return {@link ValidationResult} with a list of errors if the validation failed
     */
    ValidationResult validatePublicKey(final RSAKey publicKey);

    /**
     * Encrypts the submission data payload (json or xml) with JWE (JSON-Web-Encryption).
     *
     * @param publicKey the public encryption key the data is encrypted with
     * @param data      the {@link Data} payload that is encrypted
     * @return the JWE-encrypted {@link Data}, is empty if en error occurred
     *
     * @throws EncryptionException if the encryption fails due to an invalid key or corrupt data
     */
    Data encryptSubmissionData(final RSAKey publicKey, final Data data) throws EncryptionException;

    /**
     * Encrypts the submission attachment binary data with JWE (JSON-Web-Encryption).
     *
     * @param publicKey  the public encryption key the attachment is encrypted with
     * @param attachment the {@link Attachment} that is encrypted
     * @return the JWE-encrypted {@link Attachment}, is empty if en error occurred
     *
     * @throws EncryptionException
     */
    Attachment encryptAttachment(final RSAKey publicKey, final Attachment attachment) throws EncryptionException;

    /**
     * Authenticates the {@link Sender} against the FitConnect-Auth-API and retrieves an {@link OAuthToken}.
     *
     * @param clientId     a unique client identifier
     * @param clientSecret the applications secret key
     * @param scope        1 or more client scopes that determine if a submission is accepted by the client
     * @return {@link OAuthToken}, is empty if an error occurred
     *
     * @throws AuthenticationException
     */
    OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope) throws AuthenticationException;

    /**
     * Creates and uploads a {@link Metadata} object that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param data        {@link Data} object of the json or xml payload
     * @param attachments a list of binary {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s, is empty if an error occurred
     */
    Metadata createMetadata(final Data data, final List<Attachment> attachments) throws MetadataNotCreatedException;

    /**
     * Creates and uploads a {@link Metadata} object that contains {@link Data}, {@link Attachment} and their hashes so a subscriber
     * can verify the integrity.
     *
     * @param attachments a list of binary {@link Attachment}s
     * @return a valid {@link Metadata} object with hashed {@link Data} and {@link Attachment}s, is empty if an error occurred
     *
     * @throws MetadataNotCreatedException if there was an error during the upload or on creation of the metadata object
     */
    Metadata createMetadata(final List<Attachment> attachments) throws MetadataNotCreatedException;

    /**
     * Creates and announces a new {@link SubmissionSubmit}
     *
     * @param submission with a destinationId, a list of attachmentIds and a serviceType
     * @return the created submission
     *
     * @throws SubmissionNotCreatedException if the announcement of the submission failed due to a technical problem
     */
    SubmissionForPickup createSubmission(SubmissionSubmit submission) throws SubmissionNotCreatedException;

    /**
     * Posts the announced {@link SubmissionSubmit}
     *
     * @param submissionId      identifier of the announced submission
     * @param encryptedMetadata the encrypted metadata
     * @return the submission
     *
     * @throws SubmissionNotSentException
     */
    SubmissionForPickup sendSubmission(UUID submissionId, Metadata encryptedMetadata) throws SubmissionNotSentException;

    /**
     *
     * @param submissionId
     * @param encryptedAttachments
     * @throws AttachmentUploadException
     */
    void uploadAttachments(UUID submissionId, List<Attachment> encryptedAttachments) throws AttachmentUploadException;

    /**
     *
     * @param file of
     * @return an {@link Attachment} that contains the file content as a hashed string
     *
     * @throws AttachmentCreationException if the file could not be loaded
     */
    Attachment createAttachment(File file) throws AttachmentCreationException;

    /**
     * Retrieves the public encryption key for a destination.
     *
     * @param destinationId
     * @return the public key a submission can be encrypted with
     * @throws KeyNotRetrievedException if a technical problem occurred or the destination id is not valid
     */
    RSAKey getEncryptionKeyForDestination(UUID destinationId) throws KeyNotRetrievedException;
}
